@@ -26,26 +26,32 @@
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 from __future__ import with_statement
+import sys
+
+# are we running python 3.x ?
+PY3 = sys.version_info[0] == 3
+
 import datetime
 import logging
-import sys
 import threading
 import time
 import socket
-# import requests
-import urllib
-import urllib2
 import uuid
-import ConfigParser
 import os
 import decorator
 
 from errormator_client.ext_json import json
 from errormator_client.utils import asbool, aslist
+from errormator_client.timing import local_timing, get_local_storage
 from webob import Request
 
-# are we running python 3.x ?
-PY3 = sys.version_info[0] == 3
+if PY3:
+    import urllib
+    import configparser
+else:
+    import urllib
+    import urllib2
+    import ConfigParser
 
 DATE_FRMT = '%Y-%m-%dT%H:%M:%S'
 LEVELS = {'debug': logging.DEBUG,
@@ -58,8 +64,8 @@ log = logging.getLogger(__name__)
 
 
 class Client(object):
-    __version__ = '0.5.1'
-    __protocol_version__ = '0.3'
+    __version__ = '0.5.5'
+    __protocol_version__ = '0.3.5'
 
     def __init__(self, config=None, register_timing=True):
         """
@@ -75,7 +81,11 @@ class Client(object):
         self.config['enabled'] = asbool(config.get('errormator', True))
         self.config['server_name'] = config.get('errormator.server_name') \
                                                             or socket.getfqdn()
-        self.config['client'] = config.get('errormator.client', 'python')
+        if PY3:
+            default_client = 'python3'
+        else:
+            default_client = 'python'
+        self.config['client'] = config.get('errormator.client', default_client)
         self.config['api_key'] = config.get('errormator.api_key')
         self.config['server_url'] = config.get('errormator.server_url',
                                                'https://api.errormator.com')
@@ -106,7 +116,9 @@ class Client(object):
                                                  'session']
         user_blacklist = aslist(config.get('errormator.request_keys_blacklist',
                             config.get('errormator.bad_request_keys')), ',')
-        self.config['request_keys_blacklist'].extend(user_blacklist)
+        self.config['request_keys_blacklist'].extend(
+                                            filter(lambda x: x, user_blacklist)
+                                            )
         if config.get('errormator.bad_request_keys'):
             log.warning('errormator.bad_request_keys is deprecated use request_keys_blacklist')  # pragma: nocover
 
@@ -115,17 +127,21 @@ class Client(object):
                 'HTTP_REFERER']
         environ_whitelist = aslist(
                         config.get('errormator.environ_keys_whitelist'), ',')
-        self.config['environ_keys_whitelist'].extend(environ_whitelist)
+        self.config['environ_keys_whitelist'].extend(
+                                        filter(lambda x: x, environ_whitelist)
+                                        )
         self.config['log_namespace_blacklist'] = aslist(
                             config.get('errormator.log_namespace_blacklist',
                                        'errormator_client.client'), ',')
+        self.config['log_namespace_blacklist'] = filter(lambda x: x,
+                                        self.config['log_namespace_blacklist'])
 
         self.filter_callable = config.get('errormator.filter_callable')
         if self.filter_callable:
             try:
                 parts = self.filter_callable.split(':')
                 _tmp = __import__(parts[0], globals(), locals(),
-                                  [parts[1], ], -1)
+                                  [parts[1], ], 0)
                 self.filter_callable = getattr(_tmp, parts[1])
             except ImportError as e:
                 self.filter_callable = self.data_filter
@@ -153,7 +169,7 @@ class Client(object):
                 if k.startswith('errormator.timing'):
                     try:
                         self.config['timing'][k[18:]] = float(v)
-                    except (TypeError, ValueError), e:
+                    except (TypeError, ValueError) as e:
                         self.config['timing'][k[18:]] = False
             import errormator_client.timing
             errormator_client.timing.register_timing(self.config)
@@ -162,6 +178,7 @@ class Client(object):
                           "reports": '/api/reports',
                           "slow_reports": '/api/slow_reports',
                           "logs": '/api/logs',
+                          "request_stats": '/api/request_stats'
                           }
 
         self.report_queue = []
@@ -170,13 +187,18 @@ class Client(object):
         self.slow_report_queue_lock = threading.RLock()
         self.log_queue = []
         self.log_queue_lock = threading.RLock()
+        self.request_stats = {}
+        self.request_stats_lock = threading.RLock()
         self.uuid = uuid.uuid4()
-        self.last_submit = datetime.datetime.now()
+        self.last_submit = datetime.datetime.utcnow()
+        self.last_request_stats_submit = datetime.datetime.utcnow()
 
     def submit_data(self):
+        self.last_submit = datetime.datetime.utcnow()
         results = {'reports': False,
                    'logs': False,
-                   'slow_reports': False
+                   'slow_reports': False,
+                   'request_stats': False
                    }
         with self.report_queue_lock:
             reports = self.report_queue[:250]
@@ -191,6 +213,20 @@ class Client(object):
         results['slow_reports'] = self.api_create_submit(slow_reports,
                                                          'slow_reports')
         results['logs'] = self.api_create_submit(logs, 'logs')
+        delta = datetime.datetime.utcnow() - self.last_request_stats_submit
+        if delta >= datetime.timedelta(seconds=60):
+            with self.request_stats_lock:
+                request_stats = self.request_stats
+                self.request_stats = {}
+            stat_list = []
+            for k, v in request_stats.iteritems():
+                stat_list.append({
+                                  "server": self.config['server_name'],
+                                  "metrics": v,
+                                  "timestamp": k.isoformat()
+                                  })
+            results['request_stats'] = self.api_create_submit(stat_list, 'request_stats')
+            self.last_request_stats_submit = datetime.datetime.utcnow()
         return results
 
     def api_create_submit(self, to_send_items, endpoint):
@@ -205,11 +241,10 @@ class Client(object):
         return True
 
     def check_if_deliver(self, force_send=False):
-        delta = datetime.datetime.now() - self.last_submit
+        delta = datetime.datetime.utcnow() - self.last_submit
         if delta > self.config['buffer_flush_interval'] or force_send:
             submit_data_t = threading.Thread(target=self.submit_data)
             submit_data_t.start()
-            self.last_submit = datetime.datetime.now()
             return True
         return False
 
@@ -226,7 +261,7 @@ class Client(object):
         log.info('sending out %s entries to %s' % (len(data), endpoint,))
         try:
             req = urllib2.Request(server_url,
-                                  json.dumps(data),
+                                  json.dumps(data).encode('utf8'),
                                   headers=headers)
         except IOError as e:
             message = 'ERRORMATOR: problem: %s' % e
@@ -259,19 +294,23 @@ class Client(object):
         return structure
 
     def py_report(self, environ, traceback=None, message=None, http_status=200,
-                  start_time=None):
+                  start_time=None, request_stats={}):
         report_data, errormator_info = self.create_report_structure(environ,
                         traceback, server=self.config['server_name'],
                         http_status=http_status, include_params=True)
         report_data = self.filter_callable(report_data, 'error_report')
         url = report_data['report_details'][0]['url']
+        if not PY3:
+            url = url.decode('utf8', 'ignore')
         if start_time:
             report_data['report_details'][0]['start_time'] = start_time
+        if traceback:
+            report_data['report_details'][0]['request_stats'] = request_stats
         with self.report_queue_lock:
             self.report_queue.append(report_data)
         log.warning(u'%s code: %s @%s' % (http_status,
                             report_data.get('error_type'),
-                            url.decode('utf8', 'ignore'),))
+                            url,))
         return True
 
     def py_log(self, environ, records=None, r_uuid=None, traceback=None):
@@ -293,17 +332,20 @@ class Client(object):
                         time.gmtime(record.created)) + ('.%f' % record.msecs)
             try:
                 message = record.getMessage()
-                log_entries.append(
-                        {'log_level': record.levelname,
-                         "namespace": record.name,
-                        'message': '%s' % (message.encode('utf8') \
-                                          if isinstance(message, unicode) \
-                                          else message,),
-                        'server': self.config['server_name'],
-                        'date': time_string,
-                        'request_id': r_uuid
-                        })
-            except (TypeError, UnicodeDecodeError, UnicodeEncodeError), e:
+                log_dict = {'log_level': record.levelname,
+                            "namespace": record.name,
+                            'server': self.config['server_name'],
+                            'date': time_string,
+                            'request_id': r_uuid
+                            }
+                if PY3:
+                    log_dict['message'] = '%s' % (message)
+                else:
+                    log_dict['message'] = '%s' % (message.encode('utf8') \
+                                                  if isinstance(message, unicode) \
+                                                  else message,)
+                log_entries.append(log_dict)
+            except (TypeError, UnicodeDecodeError, UnicodeEncodeError) as e:
                 # handle some weird case where record.getMessage() fails
                 log.warning(e)
         with self.log_queue_lock:
@@ -311,15 +353,20 @@ class Client(object):
         log.debug('add %s log entries to queue' % len(records))
         return True
 
-    def py_slow_report(self, environ, start_time, end_time, records=()):
+    def py_slow_report(self, environ, start_time, end_time, records=(),
+                       request_stats={}):
         report_data, errormator_info = self.create_report_structure(environ,
                     server=self.config['server_name'], include_params=True)
         report_data = self.filter_callable(report_data, 'slow_report')
         url = report_data['report_details'][0]['url']
         report_data['report_details'][0]['start_time'] = start_time
         report_data['report_details'][0]['end_time'] = end_time
+        report_data['report_details'][0]['request_stats'] = request_stats
         report_data['report_details'][0]['slow_calls'] = []
         for record in records:
+            # we don't need that and json will barf anyways
+            del record['ignore_in']
+            del record['parents']
             r = getattr(record, 'errormator_data', record)
             report_data['report_details'][0]['slow_calls'].append(r)
         with self.slow_report_queue_lock:
@@ -327,6 +374,19 @@ class Client(object):
         log.info('slow request/queries detected: %s' % url)
         return True
 
+    def save_request_stats(self, stats):
+        with self.request_stats_lock:
+            req_time = datetime.datetime.utcnow().replace(second=0,
+                                                          microsecond=0)
+            if req_time not in self.request_stats:
+                self.request_stats[req_time] = {'main': 0, 'sql': 0,
+                                                'nosql': 0, 'remote': 0,
+                                                'tmpl': 0, 'unknown': 0,
+                                                'requests': 0}
+            self.request_stats[req_time]['requests'] += 1
+            for k, v in stats.iteritems():
+                self.request_stats[req_time][k] += v
+
     def process_environ(self, environ, traceback=None, include_params=False):
         # form friendly to json encode
         parsed_environ = {}
@@ -342,7 +402,10 @@ class Client(object):
                                 key in self.config['environ_keys_whitelist']):
                     try:
                         if isinstance(value, str):
-                            parsed_environ[key] = value.decode('utf8')
+                            if PY3:
+                                parsed_environ[key] = value
+                            else:
+                                parsed_environ[key] = value.decode('utf8')
                         else:
                             parsed_environ[key] = unicode(value)
                     except Exception as e:
@@ -374,7 +437,7 @@ class Client(object):
         parsed_environ['REMOTE_ADDR'] = remote_addr
         try:
             errormator_info['URL'] = req.url
-        except (UnicodeEncodeError, UnicodeDecodeError), e:
+        except (UnicodeEncodeError, UnicodeDecodeError) as e:
             errormator_info['URL'] = '/invalid-encoded-url'
         return parsed_environ, errormator_info
 
