@@ -2,12 +2,13 @@ import datetime
 import logging
 import pkg_resources
 import socket
+import time
 import unittest
 from errormator_client import client, make_errormator_middleware
 from errormator_client.exceptions import get_current_traceback
 from errormator_client.logger import register_logging
 from errormator_client.wsgi import ErrormatorWSGIWrapper
-
+from webob import Request
 
 fname = pkg_resources.resource_filename('errormator_client',
                                         'templates/default_template.ini')
@@ -17,7 +18,7 @@ for k, v in timing_conf.iteritems():
         timing_conf[k] = 0.000001
 
 client.Client(config=timing_conf)
-from errormator_client.timing import local_timing
+from errormator_client.timing import local_timing, get_local_storage
 
 
 def example_filter_callable(structure, section=None):
@@ -91,7 +92,8 @@ PARSED_REPORT_500 = {'traceback': u'Traceback (most recent call last):',  # this
                                                      'HTTP_CACHE_CONTROL': u'max-age=0',
                                                      'HTTP_ACCEPT_ENCODING': u'gzip, deflate'},
                                          'user_agent': u'Mozilla/5.0 (X11; Linux x86_64; rv:10.0.1) Gecko/20100101 Firefox/10.0.1',
-                                         'message': u''}],
+                                         'message': u'',
+                                         'request_stats': {}}],
                      'error_type': u'Exception: Test Exception',
                      'server': SERVER_NAME,
                      'priority': 5,
@@ -112,7 +114,8 @@ PARSED_SLOW_REPORT = {
                                                       'GET': {u'aaa': [u'1'], u'bbb': [u'2']}},
                                           'user_agent': u'',
                                           'message': u'',
-                                          'end_time': REQ_END_TIME}],
+                                          'end_time': REQ_END_TIME,
+                                          'request_stats': {}}],
                       'error_type': 'Unknown',
                       'server': SERVER_NAME,
                       'priority': 5,
@@ -125,6 +128,9 @@ class TestClientConfig(unittest.TestCase):
     def setUpClient(self, config={}):
         self.client = client.Client(config)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_empty_init(self):
         self.setUpClient()
         self.assertIsInstance(self.client, client.Client)
@@ -173,7 +179,9 @@ class TestClientConfig(unittest.TestCase):
 
     def test_default_client_name(self):
         self.setUpClient()
-        self.assertEqual(self.client.config['client'], 'python')
+
+        self.assertEqual(self.client.config['client'], 'python3' if client.PY3 \
+                         else 'python')
 
     def test_default_timeout(self):
         self.setUpClient()
@@ -386,6 +394,9 @@ class TestClientSending(unittest.TestCase):
     def setUpClient(self, config={'errormator.api_key': 'blargh!'}):
         self.client = client.Client(config)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_check_if_deliver_false(self):
         self.setUpClient()
         self.assertEqual(self.client.check_if_deliver(), False)
@@ -398,24 +409,28 @@ class TestClientSending(unittest.TestCase):
         self.setUpClient()
         self.client.py_report(TEST_ENVIRON, http_status=404)
         self.client.submit_data()
+        get_local_storage(local_timing).clear()
         self.assertEqual(self.client.report_queue, [])
 
     def test_send_error_failure(self):
         self.setUpClient()
         self.client.py_report(TEST_ENVIRON, http_status=404)
         result = self.client.submit_data()
+        get_local_storage(local_timing).clear()
         self.assertEqual(result['reports'], False)
 
     def test_send_error_io(self):
         self.setUpClient()
         self.client.py_report(TEST_ENVIRON, http_status=404)
         result = self.client.submit_data()
+        get_local_storage(local_timing).clear()
         self.assertEqual(result['reports'], False)
 
 
 class TestErrorParsing(unittest.TestCase):
     def setUpClient(self, config={}):
         self.client = client.Client(config)
+        self.maxDiff = None
 
     def test_py_report_404(self):
         self.setUpClient()
@@ -447,6 +462,7 @@ class TestErrorParsing(unittest.TestCase):
 class TestLogs(unittest.TestCase):
     def setUpClient(self, config={}):
         self.client = client.Client(config)
+        self.maxDiff = None
 
     def test_py_log(self):
         self.setUpClient()
@@ -503,43 +519,52 @@ class TestTimingHTTPLibs(unittest.TestCase):
     def setUp(self):
         self.setUpClient(timing_conf)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_urllib_URLOpener_open(self):
         import urllib
         opener = urllib.URLopener()
         opener.open("http://www.ubuntu.com/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_urllib_urlretrieve(self):
         import urllib
         urllib.urlretrieve("http://www.ubuntu.com/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_urllib2(self):
         import urllib2
         urllib2.urlopen("http://www.ubuntu.com/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_urllib3(self):
-        import urllib3
+        try:
+            import urllib3
+        except ImportError:
+            return
         http = urllib3.PoolManager()
         http.request('GET', "http://www.ubuntu.com/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_requests(self):
-        import requests
+        try:
+            import requests
+        except ImportError:
+            return
         requests.get("http://www.ubuntu.com/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_httplib(self):
         import httplib
         h2 = httplib.HTTPConnection("www.ubuntu.com")
         h2.request("GET", "/")
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
 
@@ -552,6 +577,9 @@ class TestDBApi2Drivers(unittest.TestCase):
         self.setUpClient(timing_conf)
         self.stmt = '''SELECT 1+2+3 as result'''
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_sqlite(self):
         try:
             import sqlite3
@@ -563,7 +591,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_psycopg2(self):
@@ -578,7 +606,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_pg8000(self):
@@ -593,7 +621,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_postgresql(self):
@@ -607,7 +635,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()[0]
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_mysqldb(self):
@@ -621,18 +649,21 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_oursql(self):
-        import oursql
+        try:
+            import oursql
+        except ImportError:
+            return
         conn = oursql.connect(passwd="test", user="test")
         c = conn.cursor(oursql.DictCursor)
         c.execute(self.stmt)
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_odbc(self):
@@ -646,7 +677,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_pymysql(self):
@@ -660,7 +691,7 @@ class TestDBApi2Drivers(unittest.TestCase):
         c.fetchone()
         c.close()
         conn.close()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
 
@@ -672,6 +703,9 @@ class TestMako(unittest.TestCase):
     def setUp(self):
         self.setUpClient(timing_conf)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_render(self):
         try:
             import mako
@@ -685,7 +719,7 @@ class TestMako(unittest.TestCase):
         xxxxx ${1+2} yyyyyy
         ''')
         template.render()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_render_unicode(self):
@@ -701,7 +735,7 @@ class TestMako(unittest.TestCase):
         xxxxx ${1+2} yyyyyy
         ''')
         template.render_unicode()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
     def test_template_lookup(self):
@@ -719,7 +753,7 @@ class TestMako(unittest.TestCase):
         ''')
         template = lookup.get_template("base.html")
         template.render_unicode()
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
 
@@ -728,6 +762,9 @@ class TestJinja2(unittest.TestCase):
     def setUpClient(self, config={}):
         self.client = client.Client(timing_conf)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_render(self):
         try:
             import jinja2
@@ -739,7 +776,7 @@ class TestJinja2(unittest.TestCase):
         xxxxx {{1+2}} yyyyyy
         ''')
         template.render(sleep=time.sleep)
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
 
@@ -751,6 +788,9 @@ class TestDjangoTemplates(unittest.TestCase):
     def setUp(self):
         self.setUpClient(timing_conf)
 
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
     def test_render(self):
         try:
             from django import template
@@ -765,8 +805,57 @@ class TestDjangoTemplates(unittest.TestCase):
         xxxxx {{ time }} yyyyyy
         ''')
         template.render(ctx)
-        result = local_timing._errormator.get_slow_calls()
+        stats, result = get_local_storage(local_timing).get_thread_stats()
         self.assertEqual(len(result), 1)
 
+class WSGITests(unittest.TestCase):
+
+    def setUpClient(self, config={}):
+        self.client = client.Client(timing_conf)
+
+    def tearDown(self):
+        get_local_storage(local_timing).clear()
+
+    def test_normal_request(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('Content-Type', 'text/html')])
+            return ['Hello World!']
+        req = Request.blank('http://localhost/test')
+        app = make_errormator_middleware(app, global_config=timing_conf)
+        req.get_response(app)
+        self.assertEqual(len(app.errormator_client.slow_report_queue), 0)
+
+    def test_error_request(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('Content-Type', 'text/html')])
+            raise Exception('WTF?')
+            return ['Hello World!']
+        req = Request.blank('http://localhost/test')
+        app = make_errormator_middleware(app, global_config=timing_conf)
+        app.errormator_client.config['reraise_exceptions'] = False
+        req.get_response(app)
+        self.assertEqual(len(app.errormator_client.report_queue), 1)
+
+    def test_slow_request(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('Content-Type', 'text/html')])
+            time.sleep(1.1)
+            return ['Hello World!']
+        req = Request.blank('http://localhost/test')
+        app = make_errormator_middleware(app, global_config=timing_conf)
+        req.get_response(app)
+        self.assertEqual(len(app.errormator_client.slow_report_queue), 1)
+
+    def test_logging_request(self):
+        def app(environ, start_response):
+            start_response('200 OK', [('Content-Type', 'text/html')])
+            logging.warning('test logging')
+            logging.critical('test logging critical')
+            return ['Hello World!']
+        req = Request.blank('http://localhost/test')
+        app = make_errormator_middleware(app, global_config=timing_conf)
+        req.get_response(app)
+        self.assertEqual(len(app.errormator_client.log_queue), 2)
+
 if __name__ == '__main__':
     unittest.main()  # pragma: nocover
