@@ -1,7 +1,9 @@
 import uuid
 import datetime
 from errormator_client.exceptions import get_current_traceback
-from errormator_client.timing import local_timing
+from errormator_client.timing import local_timing, get_local_storage
+from errormator_client.timing import default_timer
+from errormator_client.client import PY3
 import logging
 import sys
 
@@ -25,7 +27,7 @@ class ErrormatorWSGIWrapper(object):
         app_iter = None
         detected_data = []
         traceback = None
-        start_time = datetime.datetime.utcnow()
+        start_time = default_timer()
 
         def detect_headers(status, headers, *k, **kw):
             detected_data[:] = status[:3], headers
@@ -51,14 +53,13 @@ class ErrormatorWSGIWrapper(object):
         except Exception as e:
             if hasattr(app_iter, 'close'):
                 app_iter.close()
-            #we need that here
+            # we need that here
 
-            exc_type, exc_value, tb = sys.exc_info()
             traceback = get_current_traceback(skip=1, show_hidden_frames=True,
                                               ignore_system_exceptions=True)
             # by default reraise exceptions for app/FW to handle
             if self.errormator_client.config['reraise_exceptions']:
-                raise exc_type, exc_value, tb
+                raise
             try:
                 start_response('500 INTERNAL SERVER ERROR',
                         [('Content-Type', 'text/html; charset=utf-8')])
@@ -71,6 +72,12 @@ class ErrormatorWSGIWrapper(object):
                 return 'Server Error'
         finally:
             # report 500's and 404's
+            # report slowness
+            end_time = default_timer()
+            errormator_storage = get_local_storage(local_timing)
+            errormator_storage.thread_stats['main'] = end_time - start_time
+            stats, slow_calls = errormator_storage.get_thread_stats()
+            errormator_storage.clear()
             if traceback and self.errormator_client.config['report_errors']:
                 http_status = 500
             elif (self.errormator_client.config['report_404'] and
@@ -82,21 +89,18 @@ class ErrormatorWSGIWrapper(object):
                 self.errormator_client.py_report(environ, traceback,
                                                  message=None,
                                                  http_status=http_status,
-                                                 start_time=start_time)
-
-            # report slowness
+                    start_time=datetime.datetime.utcfromtimestamp(start_time),
+                    request_stats=stats)
+            delta = datetime.timedelta(seconds=(end_time - start_time))
             if self.errormator_client.config['slow_requests']:
                 # do we have slow calls ?
-                end_time = datetime.datetime.utcnow()
-                delta = end_time - start_time
-                records = []
-                if hasattr(local_timing, '_errormator'):
-                    for record in local_timing._errormator.get_slow_calls():
-                        records.append(record)
+                self.errormator_client.save_request_stats(stats)
                 if (delta >= self.errormator_client.config['slow_request_time']
-                    or records):
+                    or slow_calls):
                     self.errormator_client.py_slow_report(environ,
-                                    start_time, end_time, records)
+                            datetime.datetime.utcfromtimestamp(start_time),
+                            datetime.datetime.utcfromtimestamp(end_time),
+                            slow_calls, request_stats=stats)
                     # force log fetching
                     traceback = True
             if self.errormator_client.config['logging']:
